# -*- coding: utf-8 -*-
import unittest

from a3py.improved import json


class T(unittest.TestCase):

    def test__fast_json_dumps__success(self):
        self.assertEqual(
            json.fast_dumps({'a': 'b'}),
            '{"a":"b"}'
        )

    def test__fast_json_loads__success(self):
        self.assertEqual(
            json.fast_loads('[1, 2, 3]'),
            [1, 2, 3]
        )



# -*- coding: utf-8 -*-
import unittest

from a3py.simplified import case


class T(unittest.TestCase):

    def test__snake2camel__success(self):
        self.assertEqual(case.snake2camel('phone'), 'Phone')
        self.assertEqual(case.snake2camel('phone_number'), 'PhoneNumber')
        self.assertEqual(case.snake2camel('p_n'), 'PN')

    def test__camel2snake__success(self):
        self.assertEqual(case.camel2snake('Phone'), 'phone')
        self.assertEqual(case.camel2snake('PhoneNumber'), 'phone_number')
        self.assertEqual(case.camel2snake('PN'), 'p_n')



# -*- coding: utf-8 -*-
import re


def snake2camel(snake: str) -> str:
    word_list = list()
    for word in snake.split('_'):
        word_list.append(word.capitalize())
    return ''.join(word_list)


def camel2snake(camel: str) -> str:
    return re.sub(r'(?<!^)(?=[A-Z])', '_', camel).lower()



# -*- coding: utf-8 -*-
import ujson
from typing import Union

_VALID_JSON_TYPE = Union[None, str, int, float, bool, dict, list]


def fast_loads(str_json: str):
    return ujson.loads(str_json)


def fast_dumps(obj: _VALID_JSON_TYPE, ensure_ascii: bool = False) -> str:
    return ujson.dumps(obj, ensure_ascii=ensure_ascii)



# -*- coding: utf-8 -*-
import inspect
from types import ModuleType
from typing import Type, List, TypeVar
from importlib import import_module


T = TypeVar('T')


def find_all_subclasses(module: str | ModuleType, base_class: Type[T]) -> List[Type[T]]:
    if isinstance(module, str):
        module = import_module(module)

    all_class_list = list()
    for name in dir(module):
        obj = getattr(module, name)
        if inspect.isclass(obj):
            all_class_list.append(obj)

    all_subclass_list = list()
    for the_class in all_class_list:
        if issubclass(the_class, base_class) and the_class != base_class:
            all_subclass_list.append(the_class)

    return all_subclass_list



# -*- coding: utf-8 -*-
import unittest

from a3py.simplified import hashlib


class T(unittest.TestCase):

    def test__hash_md5__success(self):
        self.assertEqual(
            hashlib.hash_md5('123456'),
            'e10adc3949ba59abbe56e057f20f883e'
        )

    def test__hash_sha1__success(self):
        self.assertEqual(
            hashlib.hash_sha1('123456'),
            '7c4a8d09ca3762af61e59520943dc26494f8941b'
        )

    def test__hash_sha256__success(self):
        self.assertEqual(
            hashlib.hash_sha256('123456'),
            '8d969eef6ecad3c29a3a629280e686cf0c3f5d5a86aff3ca12020c923adc6c92'
        )

    def test__hash_sha384__success(self):
        self.assertEqual(
            hashlib.hash_sha384('123456'),
            '0a989ebc4a77b56a6e2bb7b19d995d185ce44090c13e2984b7ecc6d446d4b61ea9991b76a4c2f04b1b4d244841449454'
        )

    def test__hash_sha512__success(self):
        self.assertEqual(
            hashlib.hash_sha512('123456'),
            'ba3253876aed6bc22d4a6ff'
            '53d8406c6ad864195ed144ab5c87621b6c233b548baeae6956df346ec8c17f5ea10f35ee3cbc514797ed7ddd3145464e2a0bab413'
        )



# -*- coding: utf-8 -*-
from typing import Iterator
from copy import deepcopy
from datetime import datetime, date, timedelta


STANDARD_DATETIME_FORMAT = '%Y-%m-%d %H:%M:%S'
STANDARD_DATE_FORMAT = '%Y-%m-%d'


def datetime2str(target_datetime: datetime, str_format: str = STANDARD_DATETIME_FORMAT) -> str:
    return target_datetime.strftime(str_format)


def str2datetime(str_datetime: str, str_format: str = STANDARD_DATETIME_FORMAT) -> datetime:
    return datetime.strptime(str_datetime, str_format)


def date2str(target_date: date, str_format: str = STANDARD_DATE_FORMAT) -> str:
    return target_date.strftime(str_format)


def str2date(str_date: str, str_format: str = STANDARD_DATE_FORMAT) -> date:
    return datetime.strptime(str_date, str_format).date()


def timestamp2datetime(timestamp: int) -> datetime:
    return datetime.fromtimestamp(timestamp)


def datetime2timestamp(target_datetime: datetime) -> int:
    return int(target_datetime.timestamp())


def date_range(first_date: date, last_date: date) -> Iterator[date]:
    current_date = deepcopy(first_date)
    while current_date <= last_date:
        yield current_date
        current_date += timedelta(days=1)


def date_range_reversed(last_date: date, first_date: date) -> Iterator[date]:
    current_date = deepcopy(last_date)
    while current_date >= first_date:
        yield current_date
        current_date -= timedelta(days=1)


def build_first_datetime(day: date = None) -> datetime:
    if day is None:
        day = date.today()

    return datetime.combine(day, datetime.min.time())


def build_last_datetime(day: date = None):
    if day is None:
        day = date.today()

    return datetime.combine(day, datetime.max.time())



# -*- coding: utf-8 -*-
import os
from typing import Optional


def get_str(key: str, default: str = None) -> Optional[str]:
    v: Optional[str] = os.getenv(key, None)
    if v in ('None', ):
        v = None

    if v is None and default is not None:
        v = default
    return v


def get_int(key: str, default: int = None) -> Optional[int]:
    v: str = get_str(key)
    try:
        return int(v)
    except (TypeError, ValueError):
        return default


def get_bool(key: str, default: Optional[bool] = None) -> Optional[bool]:
    v: str = get_str(key)
    if v is None:
        return default

    if v in ('True', 'true', 't', '1'):
        return True
    else:
        return False



# -*- coding: utf-8 -*-
import unittest
import os

from a3py.simplified import env


class T(unittest.TestCase):
    test_key = 'test_key'

    def tearDown(self) -> None:
        super().tearDown()
        if os.environ.get(self.test_key, None):
            del os.environ[self.test_key]

    def test__get_str(self):
        self.assertEqual(env.get_str(self.test_key), None)
        os.environ[self.test_key] = 'None'
        self.assertEqual(env.get_str(self.test_key), None)

        v = 'abc'
        self.assertEqual(env.get_str(self.test_key, default=v), v)

        os.environ[self.test_key] = v
        self.assertEqual(env.get_str(self.test_key), v)

    def test__get_int(self):
        self.assertEqual(env.get_int(self.test_key), None)

        v = 123
        self.assertEqual(env.get_int(self.test_key, default=v), v)

        os.environ[self.test_key] = str(v)
        self.assertEqual(env.get_int(self.test_key), v)

    def test__get_bool(self):
        self.assertEqual(env.get_bool(self.test_key), None)

        v = True
        self.assertEqual(env.get_bool(self.test_key, default=v), v)

        os.environ[self.test_key] = str(v)
        self.assertEqual(env.get_bool(self.test_key), v)

        os.environ[self.test_key] = '123'
        self.assertEqual(env.get_bool(self.test_key), False)



# -*- coding: utf-8 -*-
import hashlib


def _hash_method(instance, raw: str) -> str:
    instance.update(raw.encode('utf-8'))
    return instance.hexdigest()


def hash_md5(raw: str) -> str:
    return _hash_method(hashlib.md5(), raw)


def hash_sha1(raw: str) -> str:
    return _hash_method(hashlib.sha1(), raw)


def hash_sha256(raw: str) -> str:
    return _hash_method(hashlib.sha256(), raw)


def hash_sha384(raw: str) -> str:
    return _hash_method(hashlib.sha384(), raw)


def hash_sha512(raw: str) -> str:
    return _hash_method(hashlib.sha512(), raw)



# -*- coding: utf-8 -*-
import unittest
from datetime import datetime, date, timedelta

from a3py.simplified import datetime as dt


class T(unittest.TestCase):

    def test__str2date__success(self):
        self.assertEqual(
            dt.str2date('2020-03-01'),
            date(2020, 3, 1)
        )

    def test__str2datetime__success(self):
        self.assertEqual(
            dt.str2datetime('2020-03-01 10:01:13'),
            datetime(2020, 3, 1, 10, 1, 13)
        )

    def test__date2str__success(self):
        self.assertEqual(
            dt.date2str(date(2019, 10, 3)),
            '2019-10-03'
        )

    def test__datetime2str__success(self):
        self.assertEqual(
            dt.datetime2str(datetime(2010, 12, 1, 10, 11, 13)),
            '2010-12-01 10:11:13'
        )

    def test__timestamp2datetime__success(self):
        self.assertEqual(
            dt.timestamp2datetime(1593706888),
            datetime(2020, 7, 3, 0, 21, 28)
        )

    def test__datetime2timestamp__success(self):
        self.assertEqual(
            dt.datetime2timestamp(datetime(2020, 7, 3, 0, 21, 28)),
            1593706888
        )

    def test__date_range(self):
        first_date = date(2020, 1, 1)
        last_date = date(2020, 1, 5)
        i = 0
        for day in dt.date_range(first_date, last_date):
            self.assertEqual(day, first_date + timedelta(days=i))
            i += 1

    def test__date_range_reversed(self):
        first_date = date(2020, 1, 1)
        last_date = date(2020, 1, 5)
        i = 0
        for day in dt.date_range_reversed(first_date=first_date, last_date=last_date):
            self.assertEqual(day, last_date - timedelta(days=i))
            i += 1

    def test__build_first_datetime(self):
        self.assertEqual(
            dt.datetime2str(dt.build_first_datetime(day=date(2020, 1, 1))),
            '2020-01-01 00:00:00'
        )

        self.assertTrue(
            dt.datetime2str(dt.build_first_datetime()).endswith("00:00:00")
        )

    def test__build_last_datetime(self):
        self.assertEqual(
            dt.datetime2str(dt.build_last_datetime(day=date(2020, 1, 1))),
            '2020-01-01 23:59:59'
        )
        self.assertTrue(
            dt.datetime2str(dt.build_last_datetime()).endswith("23:59:59")
        )





# -*- coding: utf-8 -*-
import unittest

from a3py.simplified import dynamic
from tests.test_simplified.dynamic import Animal, Dog, Cat, Bird, Eagle


class T(unittest.TestCase):

    def test__find_all_subclasses__success(self):
        animal_class_list = dynamic.find_all_subclasses('tests.test_simplified.dynamic', Animal)
        self.assertTrue(Animal not in animal_class_list)
        self.assertTrue(Dog in animal_class_list)
        self.assertTrue(Cat in animal_class_list)
        self.assertTrue(Bird in animal_class_list)
        self.assertTrue(Eagle in animal_class_list)

        bird_class_list = dynamic.find_all_subclasses('tests.test_simplified.dynamic', Bird)
        self.assertTrue(Animal not in bird_class_list)
        self.assertTrue(Dog not in bird_class_list)
        self.assertTrue(Cat not in bird_class_list)
        self.assertTrue(Bird not in bird_class_list)
        self.assertTrue(Eagle in bird_class_list)



# -*- coding: utf-8 -*-
import unittest

from a3py.improved import readable


class T(unittest.TestCase):

    def test__get_readable_size__success(self):
        self.assertEqual(
            readable.get_readable_size(10000),
            '9.77KB'
        )
